#
# ERM Environmental Modelling Assignement 2
# FM 28.11.2015
# reading Water Quality Time Series files
# class for water time series
# reading using read.csv
# files with 10 min values for six years
# convert to zoo object
#-----------------------------------------------
# set the system environment to English -> personal notebook has German UI 
Sys.setlocale("LC_TIME", "English") # Windows
Sys.setenv(LANG="EN") #
Sys.setenv(TZ='UTC') # Timezone UTC
# load library zoo
#
library(zoo)
#
# define a class to integrate all attributes of a time series
#
setClass("Water_Ts",representation(name="character", 
                                   unit="character",
                                   rawdata="data.frame",
                                   timeseries="zoo"))
wt       = new("Water_Ts")
wt@name  = "water temperature" 	
wt@unit  =  "[C"
at       = new("Water_Ts")
at@name  = "air temperature" 	
at@unit  = "[C]"
o2       = new("Water_Ts")
o2@name  = "oxygen content" 	
o2@unit  = "[mg/l]"
gr       = new("Water_Ts")
gr@name  = "global radiation" 	
gr@unit  = "[W/m]"
ph       = new("Water_Ts")
ph@name  = "pH-value" 	 
ph@unit  = ""
ch       = new("Water_Ts")
ch@name  = "chlorophyll-a total"
ch@unit  = "[micro g/l]"
tu       = new("Water_Ts")
tu@name  = "turbidity"
tu@unit  = "[%]"
co       = new("Water_Ts")
co@name  = "conductivity" 	
co@unit  = "[micro s]"
uv       = new("Water_Ts")
uv@name  = "light absorption"	
uv@unit  = "[%]"
ot       = new("Water_Ts")
ot@name  = "oxygen saturation" 	
ot@unit  = "[%]"

#
# reading csv files 
#
at@rawdata <- read.csv(file = "tl.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
wt@rawdata <- read.csv(file = "tw.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
gr@rawdata <- read.csv(file = "st.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
o2@rawdata <- read.csv(file = "o2.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
ph@rawdata <- read.csv(file = "ph.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
ch@rawdata <- read.csv(file = "ce.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
tu@rawdata <- read.csv(file = "tr.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
co@rawdata <- read.csv(file = "lf.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
uv@rawdata <- read.csv(file = "uv.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 
ot@rawdata <- read.csv(file = "ot.txt",sep="\t", header=F,dec=",",col.names=c("date","value","no data"), stringsAsFactors=F) 

#
# eliminate duplicated values
#
remove_duplicate_values <- function(water_ts)
{
 number = sum(duplicated(water_ts@rawdata[,1]))
  if(number!=0) 
  {message(paste(number,"duplicated values eliminated in",water_ts@name))
   water_ts@rawdata = water_ts@rawdata[!duplicated(water_ts@rawdata[,1]),]
  } 
 return(water_ts)
}
at = remove_duplicate_values(at)
wt = remove_duplicate_values(wt)
gr = remove_duplicate_values(gr)
o2 = remove_duplicate_values(o2)
ph = remove_duplicate_values(ph)
ch = remove_duplicate_values(ch)
tu = remove_duplicate_values(tu)
co = remove_duplicate_values(co)
uv = remove_duplicate_values(uv)
ot = remove_duplicate_values(ot)

#
# convert to zoo object incl. converting first column string to time values 
#
at@timeseries <- read.zoo(at@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
wt@timeseries <- read.zoo(wt@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
gr@timeseries <- read.zoo(gr@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
o2@timeseries <- read.zoo(o2@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
ph@timeseries <- read.zoo(ph@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
ch@timeseries <- read.zoo(ch@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
tu@timeseries <- read.zoo(tu@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
co@timeseries <- read.zoo(co@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
uv@timeseries <- read.zoo(uv@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")
ot@timeseries <- read.zoo(ot@rawdata[,1:2],format="%d.%m.%Y %H:%M",tz="UTC")

