#
# ERM Environmental Modelling Assignement 2
# FM 28.11.2015
# management methods
#-----------------------------------------------
library(hydroTSM)
#
# aggregate to daily values
#
daily_ts <- function(timeseries)
{
 return(aggregate(timeseries,as.Date(time(timeseries)),FUN=mean,na.rm=TRUE))
}

#
# check gaps
#
check_gaps <- function(time_series,time_step)
{
 for(it in 2:length(time_series[,1]))
  {
    delta = difftime(time(time_series[it]), time(time_series[it-1]),units="secs")
    if(delta!=time_step) 
    print(paste("                  Gap at",time(time_series[it]),time(time_series[it-1])))  
  }
}
#
# filter values in a time window for min max
#
filterTimeWindow <- function(time_series,start,end,minimum,maximum)
{
  is = which(time(time_series)==start)
  ie = which(time(time_series)==end)
#  print(paste("start",is," ",start," ",time(time_series)[is]))
#  print(paste("end",ie," ",end," ",time(time_series)[ie]))
  for(it in is:ie)
  {
    value = coredata(time_series)[it]
#    print(paste(it,value,time(time_series)[ie]))
    if(is.na(value))next;
    if(value >= maximum) {coredata(time_series)[it] <- NA}
    if(value <= minimum) {coredata(time_series)[it] <- NA}  
  }
  return(time_series)  
}

#
# prepare water temperature
#
wt_pre_processing <- function(year)
{
# extract year
  wt=window(wt@timeseries,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
                          end  =as.POSIXct(sprintf("%4d-12-31 23:59:59",year),"UTC")) 
# transform to regular time series zoo object
  wt = izoo2rzoo(wt,date.fmt="%Y-%m-%d %H:%M:%S",tstep="10 min") # hydroTSM !!
 if(year==2001)
  {  
# set NA values for outliers in October
   wt = filterTimeWindow(wt,"2001-10-12 00:00:00 UTC","2001-10-13 00:00:00 UTC",10.,50.)
# filter out of range values December
   wt = filterTimeWindow(wt,"2001-11-01 00:00:00 UTC","2001-12-31 00:00:00 UTC",0.,32.)
  }
# filling NA values
  wt = na.approx(wt)
  return(wt)
}
#
# prepare air temperature
#
at_pre_processing <- function(year)
{
  # extract year 
  at=window(at@timeseries,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
                          end  =as.POSIXct(sprintf("%4d-12-31 23:59:59",year),"UTC")) 
  # transform to regular time series zoo object
  at = izoo2rzoo(at,date.fmt="%Y-%m-%d %H:%M:%S",tstep="10 min") # hydroTSM !!
  # filling NA values
  at = na.approx(at)
  return(at)
}

#
# prepare oxygen
#
o2_pre_processing <- function(year)
{
  # extract year 
  o2=window(o2@timeseries,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
            end  =as.POSIXct(sprintf("%4d-12-31 23:59:59",year),"UTC")) 
  # transform to regular time series zoo object
  o2 = izoo2rzoo(o2,date.fmt="%Y-%m-%d %H:%M:%S",tstep="10 min") # hydroTSM !!
  # filling NA values
  o2 = na.approx(o2)
  return(o2)
}

##
# prepare water quality time series
#
gr_pre_processing <- function(year)
{
 # extract year 
 if(year==2000)
  {
    xx=window(gr@timeseries,start=as.POSIXct(sprintf("%4d-12-01 00:00:00",year-1),"UTC"),
                            end  =as.POSIXct(sprintf("%4d-01-11 23:59:59",year+1),"UTC")) 
    xx = izoo2rzoo(xx,date.fmt="%Y-%m-%d %H:%M:%S %Z",tstep="10 min") # hydroTSM !!
    xx=window(xx,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
              end  =as.POSIXct(sprintf("%4d-12-31 00:00:00",year),"UTC")) 
  }
 else
  {
   xx=window(gr@timeseries,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
                           end  =as.POSIXct(sprintf("%4d-12-31 23:59:59",year),"UTC")) 
  }
  # transform to regular time series zoo object
  xx = izoo2rzoo(xx,date.fmt="%Y-%m-%d %H:%M:%S",tstep="10 min") # hydroTSM !!
  # filling NA values
  xx = na.approx(xx,na.rm=FALSE)
  return(xx)
}
# prepare water quality time series
#
xx_pre_processing <- function(water_ts,year)
{
  # extract year 
  xx=window(water_ts@timeseries,start=as.POSIXct(sprintf("%4d-01-01 00:00:00",year),"UTC"),
                                end  =as.POSIXct(sprintf("%4d-12-31 23:59:59",year),"UTC"))
#  print(time(xx[1]))  
#  print(time(xx[length(xx)]))  
#  print(length(xx))  
  # transform to regular time series zoo object
  xx = izoo2rzoo(xx,date.fmt="%Y-%m-%d %H:%M:%S",tstep="10 min") # hydroTSM !!
#  print(time(xx[1]))  
#  print(time(xx[length(xx)]))  
#  print(length(xx))  
  # filling NA values
  xx = na.approx(xx,na.rm=FALSE)
#  print(time(xx[1]))  
#  print(time(xx[length(xx)]))  
#  print(length(xx))  
    return(xx)
}
